<?php namespace IFOAuth;

final class Facebook {

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function ifwp_plugins_loaded(){
		//ifwp_support_sessions();
		add_action('template_redirect', [__CLASS__, 'template_redirect']);
    }

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//
	// private
	//
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	private static $client = null;

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	private static function parse_userinfo($userinfo = []){
		$userinfo['parsed']['avatar_url'] = $userinfo['raw']['picture']['url'];
		$userinfo['parsed']['display_name'] = $userinfo['raw']['me']['name'];
		$userinfo['parsed']['first_name'] = $userinfo['raw']['me']['first_name'];
		$userinfo['parsed']['last_name'] = $userinfo['raw']['me']['last_name'];
		$userinfo['parsed']['locale'] = '';
		$userinfo['parsed']['oauth_id'] = $userinfo['raw']['me']['id'];
		$userinfo['parsed']['user_email'] = $userinfo['raw']['me']['email'];
		$userinfo['parsed']['user_url'] = '';
		return $userinfo;
    }

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//
	// public
	//
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function client(){
        if(null !== self::$client){
			return self::$client;
		}
		if(!class_exists('Facebook\Facebook')){
			$dir = ifwp_remote_lib('https://github.com/facebookarchive/php-graph-sdk/archive/refs/tags/5.7.0.zip', 'php-graph-sdk-5.7.0');
            if(is_wp_error($dir)){
                return $dir;
            }
			require_once($dir . '/src/Facebook/autoload.php');
		}
		$config = [];
		if(defined('IFOAUTH_FACEBOOK_APP_ID')){
			$config['app_id'] = IFOAUTH_FACEBOOK_APP_ID;
		}
		if(defined('IFOAUTH_FACEBOOK_APP_SECRET')){
			$config['app_secret'] = IFOAUTH_FACEBOOK_APP_SECRET;
		}
		$config['default_graph_version'] = 'v2.10';
		ifwp_session_start();
		try {
			self::$client = new \Facebook\Facebook($config);
		} catch(\Throwable $t){ // Executed only in PHP 7, will not match in PHP 5
			ifwp_session_destroy();
			wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $t->getMessage() . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
		} catch(\Exception $e){ // Executed only in PHP 5, will not be reached in PHP 7
			ifwp_session_destroy();
			wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $e->getMessage() . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
		}
		return self::$client;
    }

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function template_redirect(){
		global $wp;
		if('ifoauth-facebook' !== $wp->request){
			return;
		}
		$client = self::client();
		if(is_wp_error($client)){
			wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $client->get_error_message() . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
		}
		try {
			$userinfo = [
				'parsed' => [],
				'raw' => [],
			];
			$helper = $client->getRedirectLoginHelper();
			$accessToken = $helper->getAccessToken();
			$response = $client->get('/me?fields=id,email,first_name,last_name,name', $accessToken->getValue());
			$userinfo['raw']['me'] = $response->getGraphUser();
			$response = $client->get('/me/picture?redirect=0&type=large', $accessToken->getValue());
			$userinfo['raw']['picture'] = $response->getGraphNode();
		} catch(\Throwable $t){ // Executed only in PHP 7, will not match in PHP 5
			ifwp_session_destroy();
			wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $t->getMessage() . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
		} catch(\Exception $e){ // Executed only in PHP 5, will not be reached in PHP 7
			ifwp_session_destroy();
			wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $e->getMessage() . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
		}
		$state = isset($_SESSION['ifoauth_facebook_state']) ? wp_http_validate_url($_SESSION['ifoauth_facebook_state']) : false;
		ifwp_session_destroy();
		$parsed_userinfo = self::parse_userinfo($userinfo);
		do_action('ifoauth_facebook', $parsed_userinfo, (string) $state, $userinfo);
		OAuth::generic_authentication('facebook', $parsed_userinfo, (string) $state);
		/*if($state){
			//unset($_SESSION['ifoauth_facebook_state']);
			wp_safe_redirect($state);
			exit;
		}
		wp_safe_redirect(home_url());
    	exit;*/
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function url($redirect_to = ''){
		/*if(!has_action('ifoauth_facebook')){
			return ifwp_error(sprintf(__('Missing parameter(s): %s'), "'ifoauth_facebook' hook") . '.');
		}*/
		$client = self::client();
		if(is_wp_error($client)){
			return $client;
		}
		//if('' === $redirect_to and isset($_GET['redirect_to'])){
		if(!empty($_GET['redirect_to'])){
			$redirect_to = $_GET['redirect_to'];
		}
		$redirect_to = wp_http_validate_url($redirect_to);
		if($redirect_to){
			$_SESSION['ifoauth_facebook_state'] = $redirect_to;
        }
		try {
			$helper = $client->getRedirectLoginHelper();
			return $helper->getLoginUrl(site_url('ifoauth-facebook'), ['email']);
		} catch(\Throwable $t){ // Executed only in PHP 7, will not match in PHP 5
			ifwp_session_destroy();
			return ifwp_error($t->getMessage());
		} catch(\Exception $e){ // Executed only in PHP 5, will not be reached in PHP 7
			ifwp_session_destroy();
			return ifwp_error($e->getMessage());
		}
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//  add_action('ifoauth_facebook', function($userinfo){
//    if(email_exists($userinfo->email)){
//      $user = get_user_by('email', $userinfo->email);
//      $user = ifwp_signon_without_password($user->user_login);
//      if(is_wp_error($user)){
//        wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $user->get_error_message() . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
//      }
//    } else {
//      wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . ifwp_first_p(__('Unknown email address. Check again or try your username.')) . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
//    }
//  });
//
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
