<?php namespace IFOAuth;

final class Google {

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function ifwp_plugins_loaded(){
		add_action('template_redirect', [__CLASS__, 'template_redirect']);
    }

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//
	// private
	//
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    private static $client = null;

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	private static function parse_userinfo($userinfo = []){
		$userinfo['parsed']['avatar_url'] = $userinfo['raw']->picture;
		$userinfo['parsed']['display_name'] = $userinfo['raw']->name;
		$userinfo['parsed']['first_name'] = $userinfo['raw']->givenName;
		$userinfo['parsed']['last_name'] = $userinfo['raw']->familyName;
		$userinfo['parsed']['locale'] = $userinfo['raw']->locale;
		$userinfo['parsed']['oauth_id'] = $userinfo['raw']->id;
		$userinfo['parsed']['user_email'] = $userinfo['raw']->email;
		$userinfo['parsed']['user_url'] = $userinfo['raw']->link;
		return $userinfo;
    }

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//
	// public
	//
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function client(){
        if(null !== self::$client){
			return self::$client;
		}
		if(!class_exists('Google\Client')){
			switch(true){
				case is_php_version_compatible('8.0'):
					$version = '8.0';
					break;
				case is_php_version_compatible('7.4'):
					$version = '7.4';
					break;
				case is_php_version_compatible('7.0'):
					$version = '7.0';
					break;
				default:
					$version = '5.6';
			}
			$dir = ifwp_remote_lib('https://github.com/googleapis/google-api-php-client/releases/download/v2.12.1/google-api-php-client--PHP' . $version . '.zip');
			if(is_wp_error($dir)){
				return $dir;
			}
			require_once($dir . '/vendor/autoload.php');
		}
		self::$client = new \Google\Client;
		if(defined('IFOAUTH_GOOGLE_CLIENT_ID')){
			self::$client->setClientId(IFOAUTH_GOOGLE_CLIENT_ID);
		}
		if(defined('IFOAUTH_GOOGLE_CLIENT_SECRET')){
			self::$client->setClientSecret(IFOAUTH_GOOGLE_CLIENT_SECRET);
		}
		return self::$client;
    }

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function template_redirect(){
		global $wp;
		if('ifoauth-google' !== $wp->request){
			return;
		}
		if(!isset($_GET['code'])){
			wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . sprintf(__('Missing parameter(s): %s'), 'code') . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
		}
		$client = self::client();
		if(is_wp_error($client)){
			wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $client->get_error_message() . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
		}
		$client->addScope('email');
		$client->addScope('profile');
		$client->setAccessType('offline');
		$client->setPrompt('select_account consent');
		$client->setRedirectUri(site_url('ifoauth-google'));
		try {
			$userinfo = [
				'parsed' => [],
				'raw' => [],
			];
			$token = $client->fetchAccessTokenWithAuthCode($_GET['code']);
			if(array_key_exists('error', $token)){
				wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $token['error_description'] . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
			}
			$client->setAccessToken($token);
			$oauth = new \Google\Service\Oauth2($client);
			$userinfo['raw'] = $oauth->userinfo->get();
		} catch(\Throwable $t){ // Executed only in PHP 7, will not match in PHP 5
			wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $t->getMessage() . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
		} catch(\Exception $e){ // Executed only in PHP 5, will not be reached in PHP 7
			wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $e->getMessage() . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
		}
		$state = isset($_GET['state']) ? wp_http_validate_url(urldecode($_GET['state'])) : false;
		$parsed_userinfo = self::parse_userinfo($userinfo);
		do_action('ifoauth_google', $parsed_userinfo, (string) $state, $userinfo);
		OAuth::generic_authentication('google', $parsed_userinfo, (string) $state);
		/*if($state){
			wp_safe_redirect($state);
			exit;
		}
		wp_safe_redirect(home_url());
    	exit;*/
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function url($redirect_to = ''){
		/*if(!has_action('ifoauth_google')){
			return ifwp_error(sprintf(__('Missing parameter(s): %s'), "'ifoauth_google' hook") . '.');
		}*/
		$client = self::client();
		if(is_wp_error($client)){
			return $client;
		}
		$client->addScope('email');
		$client->addScope('profile');
		$client->setAccessType('offline');
		$client->setPrompt('select_account consent');
		$client->setRedirectUri(site_url('ifoauth-google'));
		//if('' === $redirect_to and isset($_GET['redirect_to'])){
		if(!empty($_GET['redirect_to'])){
			$redirect_to = $_GET['redirect_to'];
		}
		$redirect_to = wp_http_validate_url($redirect_to);
		if($redirect_to){
            $client->setState(urlencode($redirect_to));
        }
		try {
			return $client->createAuthUrl();
		} catch(\Throwable $t){ // Executed only in PHP 7, will not match in PHP 5
			return ifwp_error($t->getMessage());
		} catch(\Exception $e){ // Executed only in PHP 5, will not be reached in PHP 7
			return ifwp_error($e->getMessage());
		}
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//  add_action('ifoauth_google', function($userinfo){
//    if(email_exists($userinfo->email)){
//      $user = get_user_by('email', $userinfo->email);
//      $user = ifwp_signon_without_password($user->user_login);
//      if(is_wp_error($user)){
//        wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $user->get_error_message() . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
//      }
//    } else {
//      wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . ifwp_first_p(__('Unknown email address. Check again or try your username.')) . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
//    }
//  });
//
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
