<?php namespace IFOAuth;

final class LinkedIn {

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function ifwp_plugins_loaded(){
		add_action('template_redirect', [__CLASS__, 'template_redirect']);
    }

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//
	// private
	//
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    private static function parse_userinfo($userinfo = []){
		$country = $userinfo['raw']['me']['firstName']['preferredLocale']['country'];
		$language = $userinfo['raw']['me']['firstName']['preferredLocale']['language'];
		$locale = $language . '_' . $country;
		$first_name = $userinfo['raw']['me']['firstName']['localized'][$locale];
		$country = $userinfo['raw']['me']['lastName']['preferredLocale']['country'];
		$language = $userinfo['raw']['me']['lastName']['preferredLocale']['language'];
		$locale = $language . '_' . $country;
		$last_name = $userinfo['raw']['me']['lastName']['localized'][$locale];
		$userinfo['parsed']['avatar_url'] = $userinfo['raw']['me']['profilePicture']['displayImage~']['elements'][0]['identifiers'][0]['identifier'];
		$userinfo['parsed']['display_name'] = $first_name . ' ' . $last_name;
		$userinfo['parsed']['first_name'] = $first_name;
		$userinfo['parsed']['last_name'] = $last_name;
		$userinfo['parsed']['locale'] = $locale;
		$userinfo['parsed']['oauth_id'] = $userinfo['raw']['me']['id'];
		$userinfo['parsed']['user_email'] = $userinfo['raw']['email']['elements'][0]['handle~']['emailAddress'];
		$userinfo['parsed']['user_url'] = '';
		return $userinfo;
    }

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//
	// public
	//
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function template_redirect(){
		global $wp;
		if('ifoauth-linkedin' !== $wp->request){
			return;
		}
		if(!isset($_GET['code'])){
			wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . sprintf(__('Missing parameter(s): %s'), 'code') . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
		}
		$args = [];
		if(defined('IFOAUTH_LINKEDIN_CLIENT_ID')){
			$args['client_id'] = IFOAUTH_LINKEDIN_CLIENT_ID;
		}
		if(defined('IFOAUTH_LINKEDIN_CLIENT_SECRET')){
			$args['client_secret'] = IFOAUTH_LINKEDIN_CLIENT_SECRET;
		}
		$args['code'] = $_GET['code'];
		$args['grant_type'] = 'authorization_code';
		$args['redirect_uri'] = site_url('ifoauth-linkedin');
		$url = 'https://www.linkedin.com/oauth/v2/accessToken';
		$data = ifwp_remote_post($url, $args);
		if(is_wp_error($data)){
			wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $data->get_error_message() . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
		}
		$access_token = $data['access_token'];
		$userinfo = [
			'parsed' => [],
			'raw' => [],
		];
		$args = [
			'body' => [
                'projection' => '(elements*(handle~))',
				'q' => 'members',
            ],
			'headers' => [
				'Authorization' => 'Bearer ' . $access_token,
			],
		];
		$url = 'https://api.linkedin.com/v2/emailAddress';
		$data = ifwp_remote_get($url, $args);
		if(is_wp_error($data)){
			wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $data->get_error_message() . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
		}
		$userinfo['raw']['email'] = $data;
		$args = [
			'body' => [
                'projection' => '(id,firstName,lastName,profilePicture(displayImage~:playableStreams))',
            ],
			'headers' => [
				'Authorization' => 'Bearer ' . $access_token,
			],
		];
		$url = 'https://api.linkedin.com/v2/me';
		$data = ifwp_remote_get($url, $args);
		if(is_wp_error($data)){
			wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $data->get_error_message() . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
		}
		$userinfo['raw']['me'] = $data;
		$state = isset($_GET['state']) ? wp_http_validate_url(urldecode($_GET['state'])) : false;
		$parsed_userinfo = self::parse_userinfo($userinfo);
		do_action('ifoauth_linkedin', $parsed_userinfo, (string) $state, $userinfo);
		OAuth::generic_authentication('linkedin', $parsed_userinfo, (string) $state);
		/*if($state){
			wp_safe_redirect($state);
			exit;
		}
		wp_safe_redirect(home_url());
    	exit;*/
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function url($redirect_to = ''){
		/*if(!has_action('ifoauth_linkedin')){
			return ifwp_error(sprintf(__('Missing parameter(s): %s'), "'ifoauth_linkedin' hook") . '.');
		}*/
		$args = [];
		if(defined('IFOAUTH_LINKEDIN_CLIENT_ID')){
			$args['client_id'] = IFOAUTH_LINKEDIN_CLIENT_ID;
		}
		$args['redirect_uri'] = site_url('ifoauth-linkedin');
		$args['response_type'] = 'code';
		$args['scope'] = 'r_emailaddress r_liteprofile';
		//if('' === $redirect_to and isset($_GET['redirect_to'])){
		if(!empty($_GET['redirect_to'])){
			$redirect_to = $_GET['redirect_to'];
		}
		$redirect_to = wp_http_validate_url($redirect_to);
		if($redirect_to){
			$args['state'] = urlencode($redirect_to);
		}
		$url = 'https://www.linkedin.com/oauth/v2/authorization';
		return add_query_arg($args, $url);
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//  add_action('ifoauth_linkedin', function($userinfo){
//    if(email_exists($userinfo->email)){
//      $user = get_user_by('email', $userinfo->email);
//      $user = ifwp_signon_without_password($user->user_login);
//      if(is_wp_error($user)){
//        wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . $user->get_error_message() . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
//      }
//    } else {
//      wp_die('<h1>' . __('Something went wrong.') . '</h1><p>' . ifwp_first_p(__('Unknown email address. Check again or try your username.')) . '. <a href="' . home_url() . '">' . __('Go back') . '</a>.</p>');
//    }
//  });
//
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
