<?php namespace IFOAuth;

final class OAuth {

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function ifwp_plugins_loaded(){
		add_action('init', [__CLASS__, 'register_shortcodes']);
		add_action('template_redirect', [__CLASS__, 'template_redirect']);
    }

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//
	// private
	//
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    private static function generic_login($oauth = '', $userinfo = [], $redirect_to = ''){
		if(is_user_logged_in()){
			return;
		}
		$email = $userinfo['parsed']['user_email'];
		if(!email_exists($email)){
			return; // continue to signup
		}
		$user = get_user_by('email', $email);
		$user = ifwp_signon_without_password($user->user_login, false);
		if(is_wp_error($user)){
			wp_die($user);
		}
		do_action('ifoauth_generic_login', $user, $oauth, $userinfo, $redirect_to);
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    private static function generic_signup($oauth = '', $userinfo = [], $redirect_to = ''){
		if(is_user_logged_in()){
			return;
		}
		if(!get_option('users_can_register')){
			$message = __('<strong>Error</strong>: User registration is currently not allowed.');
			wp_die($message);
		}
		$email = $userinfo['parsed']['user_email'];
		if(email_exists($email)){
			$message = __('<strong>Error</strong>: This email is already registered. Please choose another one.');
			$message = ifwp_first_p($message);
			wp_die($message);
		}
		if(username_exists($email)){
			$message = __('<strong>Error</strong>: This username is already registered. Please choose another one.');
			$message = ifwp_first_p($message);
			wp_die($message);
		}
		$userdata = [
			'display_name' => $userinfo['parsed']['display_name'],
			'first_name' => $userinfo['parsed']['first_name'],
			'last_name' => $userinfo['parsed']['last_name'],
			'nickname' => $userinfo['parsed']['display_name'],
			'role' => get_option('default_role'),
			'user_email' => $email,
			'user_login' => $email,
			'user_pass' => wp_generate_password(),
		];
		$user_id = wp_insert_user($userdata);
		if(is_wp_error($user_id)){
			wp_die($user_id);
		}
		$user = ifwp_signon_without_password($email, false);
		if(is_wp_error($user)){
			wp_die($user);
		}
		do_action('ifoauth_generic_signup', $user, $oauth, $userinfo, $redirect_to);
		$user = ifwp_signon_without_password($user->user_login, false);
		if(is_wp_error($user)){
			wp_die($user);
		}
		do_action('ifoauth_generic_login', $user, $oauth, $userinfo, $redirect_to);
	}

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//
	// public
	//
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function btn_func($atts, $content = ''){
        $atts = shortcode_atts([
			'class' => '',
			'hide_disabled' => '',
			'oauth' => '',
			'redirect_to' => '',
			'title' => '',
        ], $atts, 'ifoauth_btn');
		$disabled = false;
		switch($atts['oauth']){
			case 'facebook':
				if(!defined('IFOAUTH_FACEBOOK_APP_ID') or !defined('IFOAUTH_FACEBOOK_APP_SECRET')){
					$disabled = true;
				}
				break;
			case 'google':
				if(!defined('IFOAUTH_GOOGLE_CLIENT_ID') or !defined('IFOAUTH_GOOGLE_CLIENT_SECRET')){
					$disabled = true;
				}
				break;
			case 'linkedin':
				if(!defined('IFOAUTH_LINKEDIN_CLIENT_ID') or !defined('IFOAUTH_LINKEDIN_CLIENT_SECRET')){
					$disabled = true;
				}
				break;
			default:
				return '';
		}
		$class = ifwp_remove_whitespaces($atts['class']);
		$classes = explode(' ', $class);
		$url = add_query_arg([
			'oauth' => $atts['oauth'],
		], site_url('ifoauth-redirect'));
		$redirect_to = !empty($atts['redirect_to']) ? urlencode($atts['redirect_to']) : '';
		if(!empty($_GET['redirect_to'])){
			$redirect_to = urlencode($_GET['redirect_to']);
		}
		if($redirect_to){
			$url = add_query_arg('redirect_to', $redirect_to, $url);
		}
		if(!in_array('btn', $classes)){
			$classes[] = 'btn';
		}
		$btn_classes = ['primary', 'secondary', 'success', 'danger', 'warning', 'info', 'light', 'dark'];
		$has_class = false;
		foreach($btn_classes as $btn_class){
			if(in_array('btn-' . $btn_class, $classes) or in_array('btn-outline-' . $btn_class, $classes)){
				$has_class = true;
				break;
			}
		}
		if(!$has_class and in_array('btn-link', $classes)){
			$has_class = true;
		}
		if(!$has_class){
			$classes[] = 'btn-primary';
		}
		if($disabled){
			if(ifwp_seems_true($atts['hide_disabled'])){
				return '';
			}
			if(!in_array('disabled', $classes)){
				$classes[] = 'disabled';
			}
		}
		$html = '<a href="' . $url . '" title="' . $atts['title'] . '" type="button" class="' . implode(' ', $classes) . '"';
		$html .= ' role="button"';
		if($disabled){
			$html .= ' aria-disabled="true" tabindex="-1"';
		}
		$html .= '>';
		$html .= trim($content);
		$html .= '</a>';
		return $html;
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function facebook_btn_func($atts, $content = ''){
		$atts = shortcode_atts([
			'class' => '',
			'hide_disabled' => '',
			'redirect_to' => '',
			'title' => '',
        ], $atts, 'ifoauth_facebook_btn');
		$atts['oauth'] = 'facebook';
        return self::btn_func($atts, $content);
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function generic_authentication($oauth = '', $userinfo = [], $redirect_to = ''){
		if(!in_array($oauth, ['facebook', 'google', 'linkedin'])){
			$message = sprintf(__('Invalid parameter(s): %s'), 'OAuth');
			wp_die($message);
		}
		self::generic_login($oauth, $userinfo, $redirect_to);
		self::generic_signup($oauth, $userinfo, $redirect_to);
		/*if(!is_user_logged_in()){
			$message = __('You are not currently logged in.');
			wp_die($message);
		}*/
		if($redirect_to){
			wp_safe_redirect($redirect_to);
			exit;
		}
		wp_safe_redirect(home_url());
    	exit;
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function google_btn_func($atts, $content = ''){
		$atts = shortcode_atts([
			'class' => '',
			'hide_disabled' => '',
			'redirect_to' => '',
			'title' => '',
        ], $atts, 'ifoauth_google_btn');
		$atts['oauth'] = 'google';
        return self::btn_func($atts, $content);
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function linkedin_btn_func($atts, $content = ''){
		$atts = shortcode_atts([
			'class' => '',
			'hide_disabled' => '',
			'redirect_to' => '',
			'title' => '',
        ], $atts, 'ifoauth_linkedin_btn');
		$atts['oauth'] = 'linkedin';
        return self::btn_func($atts, $content);
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function register_shortcodes(){
		add_shortcode('ifoauth_btn', [__CLASS__, 'btn_func']);
		add_shortcode('ifoauth_facebook_btn', [__CLASS__, 'facebook_btn_func']);
		add_shortcode('ifoauth_google_btn', [__CLASS__, 'google_btn_func']);
		add_shortcode('ifoauth_linkedin_btn', [__CLASS__, 'linkedin_btn_func']);
    }

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function template_redirect(){
		global $wp;
		if('ifoauth-redirect' !== $wp->request){
			return;
		}
		$oauth = !empty($_GET['oauth']) ? $_GET['oauth'] : '';
		$redirect_to = !empty($_GET['redirect_to']) ? urlencode($_GET['redirect_to']) : '';
		switch($oauth){
			case 'facebook':
				$url = ifoauth_facebook_url($redirect_to);
				if(is_wp_error($url)){
					wp_die($url);
				}
				break;
			case 'google':
				$url = ifoauth_google_url($redirect_to);
				if(is_wp_error($url)){
					wp_die($url);
				}
				break;
			case 'linkedin':
				$url = ifoauth_linkedin_url($redirect_to);
				if(is_wp_error($url)){
					wp_die($url);
				}
				break;
			default:
				$message = sprintf(__('Invalid parameter(s): %s'), 'OAuth') . '.';
				wp_die($message);
		}
		wp_redirect($url);
    	exit;
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

}
